# TODO

## Lessons

 * What is Scope?
  * Intro to what scope actually means
  * Scope Chain is similar to the Prototype Chain
   * Can model a naive Scope Chain using a Prototype Chain:


Notice the function `bar` is called a *closure* - It closes protectively over
its inner `var`'s, preventing anything from other scopes (including the global
scope) modifying the values.

```js
// Given the following code:

var foo = 'hi';
(function bar() {
  var zap = 'ho';
})();

// We can model it like so:
var globalScope = {};

// var foo = 'hi';
globalScope.foo = 'hi';

// function bar() { ... }
var barScope = Object.create(globalScope);

// var zap = 'ho';
barScope.zap = 'ho';

console.log(barScope.foo); // hi
console.log(barScope.zap); // ho
```

In the following case, you can see that due to lexical scoping, `quux` can't see
the variables inside `bar`'s scope, and vice versa. But, both can see the
globally scoped variable `foo`.


```js
// Given the following code:

var foo = 'hi';
(function bar() {
  var zap = 'ho';
})();
(function quux() {
  var zap = 'off to work we go';
})();

// We can model it like so:
var globalScope = {};

// var foo = 'hi';
globalScope.foo = 'hi';

// function bar() { ... }
var barScope = Object.create(globalScope);

// var zap = 'ho';
barScope.zap = 'ho';

// function quux() { ... }
var quuxScope = Object.create(globalScope);

// var zap = 'off to work we go';
quuxScope.zap = 'off to work we go';

console.log(barScope.foo);  // hi
console.log(barScope.zap);  // ho
console.log(quuxScope.zap); // off to work we go
```

Then we can see modifying a variable further up the scope affects other parts of
the code:

```js
// Given the following code:

var foo = 'hi';
(function bar() {
  var zap = 'ho';
})();
(function quux() {
  var zap = 'off to work we go';
  foo = 'yo';
})();

// We can model it like so:
var globalScope = {};

// var foo = 'hi';
globalScope.foo = 'hi';

// function bar() { ... }
var barScope = Object.create(globalScope);

// var zap = 'ho';
barScope.zap = 'ho';

// function quux() { ... }
var quuxScope = Object.create(globalScope);

// var zap = 'off to work we go';
quuxScope.zap = 'off to work we go';

// foo = 'yo';
// Note that this modifies the prototype directly (ie; the globalScope)
quuxScope.__proto__.foo = 'yo';

// Can also be written more generally as:
/*
function assignValue(variableName, value, scope) {
  while (scope !== globalScope && !scope.hasOwnProperty(variableName)) {
    scope = scope.__proto__;
  }
  scope[variableName] = value;
}
assignValue('foo', 'yo', quuxScope);
*/

console.log(globalScope.foo);  // yo
console.log(barScope.foo);     // yo
console.log(quuxScope.foo);    // yo

console.log(barScope.zap);     // ho
console.log(quuxScope.zap);    // off to work we go
```

 * Use the general `assignValue` from above to demonstrate shadowing.
